@@ -37,18 +37,6 @@ extern string GetInChI(istream& is);
 //Make an instance of the format class
 InChIFormat theInChIFormat;
 
-  // Helper for 2.3 -- is this atom a metal
-  bool IsMetal(OBAtom *atom)
-  {
-    const unsigned NMETALS = 78;
-    const int metals[NMETALS] = {
-    3,4,11,12,13,19,20,21,22,23,24,25,26,27,28,29,
-    30,31,37,38,39,40,41,42,43,44,45,46,47,48,49,50,55,56,57,58,59,60,61,62,63,
-    64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,87,88,89,90,91,
-    92,93,94,95,96,97,98,99,100,101,102,103};
-    return std::find(metals, metals+78, atom->GetAtomicNum())!=metals+78;
-  }
-
 /////////////////////////////////////////////////////////////////
 bool InChIFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
 {
@@ -145,7 +133,8 @@ bool InChIFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
     int j;
     for(j=0;j<piat->num_bonds;++j)
     {
-      pmol->AddBond(i+1, piat->neighbor[j]+1, piat->bond_type[j]);
+      if (i < piat->neighbor[j]) // Only add the bond in one direction
+        pmol->AddBond(i+1, piat->neighbor[j]+1, piat->bond_type[j]);
     }
 
     //Now use the implicit H info provided by InChI code to make explicit H in OBMol,
@@ -167,7 +156,6 @@ bool InChIFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
     }
   }
   pmol->AssignSpinMultiplicity(true); //true means no implicit H
-  pmol->DeleteHydrogens();
 
   //***@todo implicit H isotopes
   //Stereochemistry
@@ -243,6 +231,9 @@ bool InChIFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
       obErrorLog.ThrowError("InChI code", "Unsupported stereo type has been ignored.", obWarning);
     }
   }
+
+  pmol->DeleteHydrogens();
+
   // Tidy up the stereo chemistry by removing any objects that are not
   // consistent with OB's symmetry analysis
   StereoFrom0D(pmol);
@@ -287,7 +278,11 @@ bool InChIFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
 
   //Use any existing InChI, probably from an InChIformat input,
   //in preference to determining it from the structure.
-  if(!pConv->IsOption("r") && pmol->HasData("inchi"))
+  //but only if no InChI output option has been specified that would
+  //modify a standard InChI
+  if (pmol->HasData("inchi") && pConv->IsOption("r")==NULL && pConv->IsOption("a")==NULL &&
+    pConv->IsOption("s")==NULL && pConv->IsOption("X")==NULL && pConv->IsOption("F")==NULL &&
+    pConv->IsOption("M")==NULL)
   {
     //All origins for the data are currently acceptable.
     //Possibly this may need to be restricted to data with a local origin.
@@ -348,7 +343,7 @@ bool InChIFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
             stereo = OBStereo::DownBond;
           else if (pbond->IsWedgeOrHash())
             stereo = OBStereo::UnknownDir;
-        }
+        } 
         else if (from_cit!=from.end()) { // It's a stereo bond
           stereo = updown[pbond];
         }
@@ -381,7 +376,7 @@ bool InChIFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
       strcpy(iat.elname,etab.GetSymbol(patom->GetAtomicNum()));
       iat.num_bonds = nbonds;
       //Let inchi add implicit Hs unless the atom is known not to have any
-      iat.num_iso_H[0] = patom->HasNoHForced() || IsMetal(patom) ? 0 : -1;
+      iat.num_iso_H[0] = patom->HasNoHForced() || patom->IsMetal() ? 0 : -1;
       if(patom->GetIsotope())
       {
         iat.isotopic_mass = ISOTOPIC_SHIFT_FLAG +
